#ifndef     _H_LINKED_LIST
#define     _H_LINKED_LIST

#ifndef    lint
static const char linked_list_sccsid[]= "%W% %G% %U%: Component-%Y% %Q%";
#endif   //  lint


#include   "cpp_entry.h"
#include   "Listconsts.h"
#include   "ListImplSpec.h"
#include   "ListImpl.h"


// LinkedList.h
// This file defines the interface to the linked list component. The
// implementation is clearly separated. The interface accepts different
// types of elements and also allows the user to specify the required
// implementation type. At present only the doubly linked list and the
// locked doubly linked list are supported. The locked implementations
// are only supported for kernel clients.



template  <class Type>
  class   ListIterSecIntf ;

template  <class Type>
class    LinkedListBase : public virtual CLASS
 {
   public :
      LinkedListBase() { } ;
      ~LinkedListBase() { } ;
      virtual Type *remove_from_list(Type *) { return NULL ; }  ;
      virtual void merge_lists(LinkedListBase<Type> *,ListElemPriority){ } ;
      virtual void queue(Type *,ListElemPriority ) { }  ;
      virtual Type *dequeue(void) { return NULL ; }  ;

      virtual ListImplAncestor *ret_impl() { return NULL ; }
      RTTI_SCAFFOLDING_DECL_TEMPL(LinkedListBase<Type>)

 } ; // At present just a place holder to make everything work with cfront

// This class defines the second interface for lists. The only function
// that it supports at present it the ability to choose a particular
// implmentation. In reality it is not necessary to have this as a
// different second interface - since, it is always possible to have the
// specification of the implementation dierctly during object instantiation
// ie: LinkedList<class Object_Type, ImplSpec Implementation_spec> *foo=...
// - however, in any non-trivial case more complicated implementation
// advice should not complicate the direct interface. Hence, in this
// vein - the second interface is used to support the specification of
// the implementation category, too.


// The following class inherits from the LinkedList class and specializes
// on it. Since, it inherits from the LinkedList class, it has the
// oppurtunity to specialize them even further.

template <class Type>
class  ListSecInterface :  public LinkedListBase<Type>
{
 friend    class   ListIterSecIntf<Type> ;

 public :
    ListSecInterface (int ) ;
    ListSecInterface() ;
    virtual ~ListSecInterface () ;

    // additional second interface introduced here
    virtual void set_impl (int ) ;
    virtual int  get_impl ( void ) ;

    // support for the first interface stuff
    virtual Type *remove_from_list(Type *);
    virtual void merge_lists(LinkedListBase<Type> *,ListElemPriority pri =
							     REGULAR) ;
    virtual void queue(Type *,ListElemPriority pri = REGULAR);
    virtual Type *dequeue(void) ;
    virtual ListImplAncestor *ret_impl() { return impl ; }
    RTTI_SCAFFOLDING_DECL_TEMPL(ListSecInterface<Type>)


 // internal representation
 private :
    ListImplSpec       impl_type ; // type of the implementation
    ListImplAncestor  *impl ; // the representation
    ListImplAncestor  *meta ;  // make sure that this is not static
} ; // specification for the second interface for lists



// The actual declaration of the LInked List interface
template  <class Type>
class   LinkedList : public LinkedListBase<Type>
{
public :
 LinkedList() : sec(new ListSecInterface<Type>)  { } ;
 virtual ~LinkedList() { delete sec ; }   ;
 virtual Type *remove_from_list(Type *t) {return sec->remove_from_list(t);};

 virtual void merge_lists(LinkedListBase<Type> *l,ListElemPriority pri =
			     REGULAR) { sec->merge_lists(l,pri);}  ;
 virtual void queue(Type *i,ListElemPriority pri = REGULAR)
	 {sec->queue(i,pri);} ;
 virtual Type *dequeue(void) { return (sec->dequeue());} ;
 virtual ListImplAncestor *ret_impl() { return sec->ret_impl() ; }
 virtual ListSecInterface<Type> *operator->() { return sec ; }
 RTTI_SCAFFOLDING_DECL_TEMPL(LinkedList<Type>)

private :
  LinkedList (const LinkedList<Type>& ) { } ; // hide it
  ListSecInterface<Type> *sec ; // a pointer to the second interface
} ; // generic interface to linked lists

template  <class Type>
class  ListIteratorBase : virtual public CLASS
 {
  public :
    ListIteratorBase() { } ; // hide it
    ListIteratorBase(LinkedList<Type> &) { };
    ListIteratorBase(LinkedList<Type> *) { }  ;
    virtual ~ListIteratorBase() { }  ;
    virtual Type  *operator()() { return NULL ; }  ;
    virtual void  reset(void) { }  ;
    RTTI_SCAFFOLDING_DECL_TEMPL(ListIteratorBase<Type>)
 } ; // generic base class for Linked List iterator

template  <class Type>
class  ListIterSecIntf :   public  ListIteratorBase<Type>
{
 public :
    ListIterSecIntf(LinkedList<Type> &) ;
    ListIterSecIntf(LinkedList<Type> *) ;
    virtual ~ListIterSecIntf() ;

    // additional second interface related stuff goes here
    virtual void set_impl (int ) ;

    // operations pulled in from the first interface
    virtual Type  *operator()() ;
    virtual void  reset(void) ;
    RTTI_SCAFFOLDING_DECL_TEMPL(ListIterSecIntf<Type>)

 // internal representation
 private :
    ListImplSpec       impl_type ; // type of the implementation
    ListIterAncestor  *impl ; // the representation
    ListIterAncestor  *meta ;  // make sure that this is not static

} ;


template  <class Type>
class  ListIterator : public  ListIteratorBase<Type>
 {
    friend  LinkedList<Type> ;

  public :
    ListIterator() : sec(0), list(0) { } ;
    ListIterator(LinkedList<Type> &x)
       : sec(new ListIterSecIntf<Type> (x)), list(&x) { } ;
    ListIterator(LinkedList<Type> *x)
       : sec(new ListIterSecIntf<Type> (x)), list(x) { } ;
    virtual ~ListIterator() ;
    virtual Type  *operator()() ;
    virtual void  reset(void) ;
    virtual ListIterSecIntf<Type> *operator->() { return sec ; } ;
    RTTI_SCAFFOLDING_DECL_TEMPL(ListIterator<Type>)

  private :
    ListIterSecIntf<Type>    *sec ;
    LinkedList<Type>     *list ;
 } ; // generic interface to linked list iterator


#endif    // _H_LINKED_LIST
