#ifndef lint
  static const char sccs_id[]= "%W% %G% %U%: Component-%Y% %Q%" ;
#endif

#include    "cpp_entry.h"

// Declare a global variable to handle problems with static initialization
// orders

/*************
  Member functions for the Base  Metaclass
************/
impl_table_t      init_table = { 0 } ;

// Allows for the registration of a new type. This ensures that the
// type in question is not alreay registered and that there is enough
// space left for a new type to be added in the meta-object.

RTTI_SCAFFOLDING_IMPL1(BaseMeta, CLASS)

BaseMeta::BaseMeta()
{
  table = &init_table ;
  if ( !table->entries )
      (void ) alloc_entries() ;
} // constructor

BaseMeta::~BaseMeta()
{
   table = NULL ;
} // destructor

impl_entry   *
BaseMeta::alloc_entries ( void )
{
  // allocate memory for the table itself.
   impl_entry  *temp_entry =
	    new impl_entry [table->size += BASE_TYPE_INCR] ;

  // if memory did actually get allocated and there existed some
  // stuff in the table - then do the required housekeeping.
  if ( table->entries && temp_entry )
    {
     for ( int i = 0 ; i != table->index ; i++ )
	  temp_entry[i] = table->entries[i] ;

      delete (table->entries)  ;
      table->entries = temp_entry ;
    } // copy stuff into the new table, if necessary

  // set the pointer anyway
   else if ( temp_entry )
      table->entries = temp_entry ;

  // return a pointer to the table - this also acts as an error-code.
  return ( temp_entry ) ;

} // allocate entries

void
BaseMeta::register_type(ImplSpec *type,BaseMeta *metaobj)
{
  // make sure that there is enough space
  if ( table->index  == table->size )
   {
     (void) alloc_entries() ;
   } // allocate a new blk, if necessary

  // search for the appropriate type
  impl_entry *info = find_info(type ) ;

 // make sure that the information pointer is valid
 if (info)
  {
   if ( !info->hdlr )
     info->hdlr = metaobj ;
   return ;
  } // found a match

  table->entries[table->index].type = type ;
  table->entries[table->index++].hdlr = metaobj ;

} // register a child type

// removes a child type - ensures that the type is actually valid.
void
BaseMeta::unregister_type (ImplSpec *type)
{
  // search for the appropriate type
  impl_entry *info = find_info(type ) ;

 // make sure that the information pointer is valid
 if (info)
  {
   info->hdlr = NULL ;
  } // found a match
} // unregister a particular implementation type


// Calls the instantiate function of the meta-object of the child type.
// Returns NULL if the type does not have a valid handler.
CLASS   *
BaseMeta::instantiate(ImplSpec *type )
{
 impl_entry *info = find_info(type ) ;


 // Make sure that the hdlr exists
 if (info)
    return ( info->hdlr->instantiate(type) ) ;

  // if we get till here - return NULL
  return ( NULL ) ;

} // allocate the appropriate type of object


impl_entry *
BaseMeta::find_info(ImplSpec *type )
{

 // loop thru all entries - a better way must be found.
  for ( int i = 0 ; i != table->index ; i++ )
   {
    if ( ( *type == *(table->entries[i].type) ) )
	  return ( &(table->entries[i]) ) ;
   } // make sure that type is registered

  // if we get till here - return NULL
  return ( NULL  ) ;
} // locate the info for the given type
