#ifndef lint
static const char sccs_id[]= "%W% %G% %U%: Component-%Y% %Q%" ;
#endif

#include   "cpp_entry.h"

RTTI_SCAFFOLDING_IMPL1(ImplSpec,CLASS)

impl_map_t     global_map = { 0 } ;

void
ImplSpec::init_tab ( void )
{
  table = &global_map ;
  if ( !table->entries )
      (void ) alloc_entries() ;
} // a private initializartion routine

ImplSpec::ImplSpec()
{
 init_tab() ;
} // constructor for the class

ImplSpec::ImplSpec(const char *n)
{
 // Trace    z("ImplSpec", "ImplSpec") ;
 init_tab() ;
 impl_map_entry_t *t ;
 if ( !(t=find_name(n)) )
    t = register_name(n) ;

 id = t->uid ;
} // constructor that gets called from  derived class usually

ImplSpec::~ImplSpec()
{
  table = NULL ;
} // destructor for the class

impl_map_entry_t *
ImplSpec::find_name(const char *nm )
{
  for ( int i = 0 ; i != table->index ; i++ )
   {
     if ( !strcmp(table->entries[i].name,nm) )
	return ( &(table->entries[i]) ) ;
   } // search for the given name

  // if we get till here - return NULL
     return ( NULL ) ;
}

int
ImplSpec::is_index_valid(void )
{
 if ( table->index >= MAX_UID )
   return ( 0 ) ;
 return ( 1 ) ;
} // make sure that the given index is valid


impl_map_entry_t   *
ImplSpec::register_name(const char *nm )
{
 if ( !is_index_valid() )
    return (NULL) ;

 if ( table->index == table->size )
    (void ) alloc_entries() ;

 table->entries[table->index].name = nm ;
 table->entries[table->index].uid = table->index ;
 table->index++ ;
 return ( &(table->entries[table->index - 1])) ;
} // register a given name

impl_map_entry_t   *
ImplSpec::alloc_entries ( void )
{
  // allocate memory for the table itself.
   impl_map_entry_t  *temp_entry =
	    new impl_map_entry_t [table->size += IMPL_TABLE_INCR] ;

  // if memory did actually get allocated and there existed some
  // stuff in the table - then do the required housekeeping.
  if ( table->entries && temp_entry )
    {
     for ( int i = 0 ; i != table->index ; i++ )
	  temp_entry[i] = table->entries[i] ;

      delete (table->entries)  ;
      table->entries = temp_entry ;
    } // copy stuff into the new table, if necessary

  // set the pointer anyway
   else if ( temp_entry )
      table->entries = temp_entry ;

  // return a pointer to the table - this also acts as an error-code.
  return ( temp_entry ) ;

} // allocate entries




/*************
  Member functions for the BaseImplSpec class - just trivial operators
************/
RTTI_SCAFFOLDING_IMPL1(BaseImplSpec,ImplSpec)

BaseImplSpec::BaseImplSpec(int  val, const char *nm )
  : type(val) , ImplSpec(nm)
{
   type = ImplSpec::getuid(val) ;
} // constructor

int
BaseImplSpec::operator== (ImplSpec  &x)
{
   ImplSpec   *imp = &x ;
   BaseImplSpec *y = BaseImplSpec::_narrow(imp) ;
   return (y ? (type == y->type) : 0 );
} // tests for equality

int
BaseImplSpec::operator== (ImplSpec  *x)
{
 BaseImplSpec *y = BaseImplSpec::_narrow(x) ;
 return (y ? (type == y->type) : 0 );
} // testst for equality

int
BaseImplSpec::operator== (int  &x)
{
  return (type == x) ;
} // tests for equality

int
BaseImplSpec::operator!= (ImplSpec &x)
{
 ImplSpec   *imp = &x ;
 BaseImplSpec *y = BaseImplSpec::_narrow(imp) ;
 return (y ? (type != y->type) : 0 );
} // tests for inequality

int
BaseImplSpec::operator!= (ImplSpec *x)
{
 BaseImplSpec *y = BaseImplSpec::_narrow(x) ;
 return (y ? (type != y->type) : 0 );
} // tests for inequality

int
BaseImplSpec::operator!= (int &x)
{
 return (type != x);
} // tests for inequality

ImplSpec &
BaseImplSpec::operator = (int x)
{
   type = (x & 0xffff0000) ? x : ImplSpec::getuid(x) ;
   return ( *this )  ;
} // performs assignment

ImplSpec &
BaseImplSpec::operator = (ImplSpec &x)
{
 ImplSpec   *imp = &x ;
 BaseImplSpec *y = BaseImplSpec::_narrow(imp) ;

 if ( y )
   {
    int   i = (int) y->type ;
    type = (i & 0xffff0000) ? i : ImplSpec::getuid(i) ;
   } // make sure y is not NULL

 return ( *this ) ;
} // performs assignment

BaseImplSpec::operator int ()
{
 return type ;
} // user-defined type conversion
