#ifndef lint
  static const char sccs_id[]= "%W% %G% %U%: Component-%Y% %Q%" ;
#endif

#ifdef    _KERNEL
#include   <sys/device.h>
#endif    // _KERNEL
#include   "cpp_entry.h"
#include   "ListImpl.h"
#include   "ListImplDouble.h"
#include   "ListImplDblock.h"

/***********
 BINDINGS FOR META-OBJECTS AND SEPARATION OF INTERFACE FROM IMPLEMENTATION
***********/
// initialize the static variables in the meta-object
int    DoublyLinkLockMeta::count = 0 ;
int    DoublyLinkLockIterMeta::count = 0 ;
// create a meta-object for doubly linked lists and their iterators
DoublyLinkLockMeta  *DoublyLinkLockImpl::meta = new DoublyLinkLockMeta ;
DoublyLinkLockIterMeta *DoublyLinkLockIter::meta =
					 new  DoublyLinkLockIterMeta;

RTTI_SCAFFOLDING_IMPL1(DoublyLinkLockImpl,DoublyLinkedImpl)
RTTI_SCAFFOLDING_IMPL1(DoublyLinkLockMeta,ListMeta)
RTTI_SCAFFOLDING_IMPL1(DoublyLinkLockIterMeta,IterMeta)
RTTI_SCAFFOLDING_IMPL1(DoublyLinkLockIter,DoublyLinkedIter)

/**********
 Member functions for the Doubly linked list Meta-object
**********/

// a Non-trivial constructor which ensures that the first call registers
// the doubly linked type and this meta-object as the handler.
DoublyLinkLockMeta::DoublyLinkLockMeta()
{

  if ( count++ == 0 )
   {
     register_type(new ListImplSpec(LOCKED_DOUBLE), this);
   } // register the class
} // register the class

// destructor that handles unregistering of the class - if required
DoublyLinkLockMeta::~DoublyLinkLockMeta()
{
  if ( --count == 0 )
      unregister_type(new ListImplSpec (LOCKED_DOUBLE))  ;
} // unregister the class if required

// Handles the creation of doubly linked lists
CLASS *
DoublyLinkLockMeta::instantiate(ImplSpec *)
{
   DoublyLinkLockImpl    *ret = new DoublyLinkLockImpl ;
   return (ret) ;
}  // create a Doubly linked List object implementation

/**********
 Member functions for the Doubly linked lock list iterator Meta-object
**********/

// a Non-trivial constructor which ensures that the first call registers
// the doubly linked iter and this meta-object as the handler.
DoublyLinkLockIterMeta::DoublyLinkLockIterMeta()
{
  if ( count++ == 0 )
   {
     register_type(new ListImplSpec (LOCKED_DOUBLE_ITER), this);
   } // register the class
} // register the class

// destructor that handles unregistering of the class - if required
DoublyLinkLockIterMeta::~DoublyLinkLockIterMeta()
{
  if ( --count == 0 )
      unregister_type(new ListImplSpec(LOCKED_DOUBLE_ITER))  ;
} // unregister the class if required

// Handles the creation of doubly linked list iterators
ListIterAncestor *
DoublyLinkLockIterMeta::instantiate(ImplSpec *, ListImplAncestor *x)
{
   return new DoublyLinkLockIter(x) ;
}  // create a Doubly linked List Iterator object implementation

// Handles the creation of doubly linked list iterators
ListIterAncestor *
DoublyLinkLockIterMeta::instantiate(ImplSpec *, ListImplAncestor &x)
{
   return new DoublyLinkLockIter(x) ;
}  // create a Doubly linked List Iterator object implementation


/***********
   Member functions of the Doubly Linked List Implementation
***********/

DoublyLinkLockImpl::DoublyLinkLockImpl() : list_lock(0)
{
#ifdef   _KERNEL
   list_lock = LOCK_AVAIL ;
#endif    // _KERNEL
}

void *
DoublyLinkLockImpl::dequeue(void )
{
#ifdef     _KERNEL
   int lock_type = lockl(&list_lock, LOCK_SHORT) ;
   void *ret = DoublyLinkedImpl::dequeue() ;
   if ( lock_type != LOCK_NEST )
      unlockl(&list_lock) ;
#else
    // cerr  << "Dequeue:  Obtained Lock " << endl ;
    void *ret = DoublyLinkedImpl::dequeue() ;
#endif   // _KERNEL
    return ( ret ) ;
} // remove the first item

void
DoublyLinkLockImpl::queue(void *item, ListElemPriority pri)
{
#ifdef     _KERNEL
   int lock_type = lockl(&list_lock, LOCK_SHORT) ;
   DoublyLinkedImpl::queue(item,pri) ;
   if ( lock_type != LOCK_NEST )
      unlockl(&list_lock) ;
#else
    // cerr  << "Queue: Obtained Lock " << endl ;
    DoublyLinkedImpl::queue(item,pri) ;
#endif   // _KERNEL
} // queue an item

ListImplAncestor *
DoublyLinkLockImpl::merge_lists(ListImplAncestor *l,ListElemPriority p)
{
#ifdef     _KERNEL
   int lock_type = lockl(&list_lock, LOCK_SHORT) ;
   ListImplAncestor *ret = DoublyLinkedImpl::merge_lists(l,p) ;
   if ( lock_type != LOCK_NEST )
      unlockl(&list_lock) ;
#else
    cerr  << "Merge_lists:  Obtained Lock " << endl ;
    ListImplAncestor *ret = DoublyLinkedImpl::merge_lists(l,p) ;
#endif   // _KERNEL
    return ( ret ) ;
} // merge two lists - throw away the list passed to this


void *
DoublyLinkLockImpl::remove_specific_elem(void  *item)
{
#ifdef     _KERNEL
   int lock_type = lockl(&list_lock, LOCK_SHORT) ;
   void *ret = DoublyLinkedImpl::remove_specific_elem(item) ;
   if ( lock_type != LOCK_NEST )
      unlockl(&list_lock) ;
#else
   // cerr  << "Remove_specific_elem:  Obtained Lock " << endl ;
   void *ret = DoublyLinkedImpl::remove_specific_elem(item) ;
#endif   // _KERNEL

    return ( ret ) ;
} // search and remove a specific element












