#ifndef lint
  static const char sccs_id[]= "%W% %G% %U%: Component-%Y% %Q%" ;
#endif

#include   "cpp_entry.h"
#include   "ListImpl.h"
#include   "ListImplDouble.h"

/***********
 BINDINGS FOR META-OBJECTS AND SEPARATION OF INTERFACE FROM IMPLEMENTATION
***********/
// initialize the static variables in the meta-object
int    DoublyLinkedMeta::count = 0 ;
int    DoublyLinkedIterMeta::count = 0 ;
// create a meta-object for doubly linked lists and their iterators
DoublyLinkedMeta  *DoublyLinkedImpl::meta = new DoublyLinkedMeta ;
DoublyLinkedIterMeta  *DoublyLinkedIter::meta = new DoublyLinkedIterMeta ;

RTTI_SCAFFOLDING_IMPL1(DoubleNode,CLASS)
RTTI_SCAFFOLDING_IMPL1(DoublyLinkedImpl,ListImplAncestor)
RTTI_SCAFFOLDING_IMPL1(DoublyLinkedMeta,ListMeta)
RTTI_SCAFFOLDING_IMPL1(DoublyLinkedIterMeta,IterMeta)
RTTI_SCAFFOLDING_IMPL1(DoublyLinkedIter,ListIterAncestor)

/**********
 Member functions for the Doubly linked list Meta-object
**********/

// a Non-trivial constructor which ensures that the first call registers
// the doubly linked type and this meta-object as the handler.
DoublyLinkedMeta::DoublyLinkedMeta()
{

  if ( count++ == 0 )
   {
     register_type(new ListImplSpec(DOUBLE), this);
   } // register the class
} // register the class

// destructor that handles unregistering of the class - if required
DoublyLinkedMeta::~DoublyLinkedMeta()
{
  if ( --count == 0 )
      unregister_type(new ListImplSpec (DOUBLE))  ;
} // unregister the class if required

// Handles the creation of doubly linked lists
CLASS *
DoublyLinkedMeta::instantiate(ImplSpec *)
{
   DoublyLinkedImpl    *ret = new DoublyLinkedImpl ;
   return (ret) ;
}  // create a Doubly linked List object implementation

/**********
 Member functions for the Doubly linked list iterator Meta-object
**********/

// a Non-trivial constructor which ensures that the first call registers
// the doubly linked iter and this meta-object as the handler.
DoublyLinkedIterMeta::DoublyLinkedIterMeta()
{
  if ( count++ == 0 )
   {
     register_type(new ListImplSpec (DOUBLE_ITER), this);
   } // register the class
} // register the class

// destructor that handles unregistering of the class - if required
DoublyLinkedIterMeta::~DoublyLinkedIterMeta()
{
  if ( --count == 0 )
      unregister_type(new ListImplSpec(DOUBLE_ITER))  ;
} // unregister the class if required

// Handles the creation of doubly linked list iterators
ListIterAncestor *
DoublyLinkedIterMeta::instantiate(ImplSpec *, ListImplAncestor *x)
{
   return new DoublyLinkedIter(x) ;
}  // create a Doubly linked List Iterator object implementation

// Handles the creation of doubly linked list iterators
ListIterAncestor *
DoublyLinkedIterMeta::instantiate(ImplSpec *, ListImplAncestor &x)
{
   return new DoublyLinkedIter(x) ;
}  // create a Doubly linked List Iterator object implementation

/***********
  Member functions for the Doubly linked list Iterator implementation
***********/
DoublyLinkedIter::DoublyLinkedIter(ListImplAncestor *x)
{
   list = DoublyLinkedImpl::_narrow(x) ;
   current_node = list->first ;
} // constructor for iterator impl

DoublyLinkedIter::DoublyLinkedIter(ListImplAncestor &x)
{
   list = DoublyLinkedImpl::_narrow(&x) ;
   current_node = list->first ;
} // constructor for iterator impl

void
DoublyLinkedIter::reset(void)
{
 current_node = list->first ;
} // reset the current node pointer

void *
DoublyLinkedIter::operator() ()
{
  DoubleNode   *n = current_node ;
  current_node =  (current_node) ? (current_node->get_next()) : NULL ;
  return ( n ? (n->get_elem()) : NULL ) ;
} // the main iteration operator

/***********
  Member functions for Node objects of the Doubly linked list
***********/
// constructor
DoubleNode::DoubleNode(void *e)
 : prev(0), next(0), elem(e)
{

}

// destructor
DoubleNode::~DoubleNode()
{
  if ( elem )
     delete elem ;
} // destructor

void
DoubleNode::do_link(DoubleNode *before, DoubleNode *after)
{
  // the first pointer may be NULL - simplifies things
  if (before == NULL)
    {
      // both first and last are NULL - just return
      if ( after == NULL)
	 return ;
      // set the pointers in the node and the after node
      next = (after ? after : next ) ;
      after->prev = this ;
    } // the previous element is NULL
  // the first node is NOT null
  else
    {
      // But the lst node is ...
      if ( after == NULL)
       {
	 before->next = this ;
	 prev = ( before ? before : prev)  ;
       } // this is the last element
      // Both before and after are valid .
      else
       {
	before->next = this ;
	prev = ( before ? before : prev ) ;
	next = ( after ? after : next )  ;
	after->prev = this ;
       } // set it right in the middle
    } // the previous element is certainly not NULL

} // routine that actually does the linking

void
DoubleNode::linkafter(DoubleNode *node)
{
 do_link(node, (node ? node->next : NULL)) ;
} // link an element in between two others

void
DoubleNode::linkbefore(DoubleNode *node )
{
  do_link((node ? node->prev : NULL), node) ;

} // attach before the given node

void *
DoubleNode::unlink( void )
{
  DoubleNode    *before = prev ;
  DoubleNode    *after = next ;
  void   *ret = elem ;

  prev = NULL ;
  next = NULL ;

  if ( !before )
   {
    // handle the trivial case first
    if ( after )
      after->prev = NULL ;
   } // before == NULL
  else
    {
      before->next = after ;
      if ( after )
	  after->prev = before ;
    } // before != NULL

  elem = NULL ;
  return (ret) ;
} // remove this link


/***********
   Member functions of the Doubly Linked List Implementation
***********/

void *
DoublyLinkedImpl::dequeue(void )
{

 DoubleNode  *ret ;

 if ((ret=first))
  {
    if (!(last = (first->get_next() ? last : NULL)))
       first = last ;
    else
       first = first->get_next() ;
    void *val = ret->unlink() ;
    delete ret ;
    return ( val) ;
  } // list is not empty

 return ( NULL ) ;
} // remove the first item

void
DoublyLinkedImpl::queue(void *item, ListElemPriority pri)
{
  DoubleNode    *node = new DoubleNode (item) ;

  if ( !first )
    first = last = node ;
  else if ( pri == HIGH)
    {
     node->linkbefore(first) ;
     first = node ;
    } // attach to begining of the queue
  else
    {
      node->linkafter(last) ;
      last = node ;
    } // attach to the end of the queue
} // queue an item

ListImplAncestor *
DoublyLinkedImpl::merge_lists(ListImplAncestor *listA,ListElemPriority pri)
{
  // get the appropriate pointer
  DoublyLinkedImpl *list = DoublyLinkedImpl::_narrow(listA) ;

   // do all this only if the passed list is not empty
   if (list->first)
     {
       // if the current list is empty - this become trivial
       if ( !first)
	 {
	   first = list->first ;
	   last = list->last ;
	 } // this list is empty at present - trivial case
       // attach the given list in front
	else if ( pri == HIGH)
	  {
	   first->linkafter(list->last) ;
	   first = list->first ;
	  } // attach this list to the bac of the passed list
       // attach the given list at the end
	else
	  {
	   last->linkbefore(list->first) ;
	   last = list->last ;
	  } // simply attach the list at the end

       // the pointers in the passed list must be set to NULL
       list->first = list->last = NULL ;
     } // make sure that the second list is not empty

   // return a pointer to myself
   return ( this ) ;

} // merge two lists - throw away the list passed to this


void *
DoublyLinkedImpl::remove_specific_elem(void  *item)
{
 DoubleNode *n  = search_for_elem(item);
 void    *ret = NULL  ;
 DoubleNode *t_first = first , *t_last = last  ;

 // remove the node if possible
 if (n)
  {
   if (n == first )
     t_first = (first == last) ? (t_last = n->get_next()) : (n->get_next());
   else if ( n == last )
     t_last = n->get_prev() ;

    ret = (void *)  n->unlink() ;
    delete n ;
    first = t_first ;
    last = t_last ;
  } // do the housekeeping - make sure that the node is valid

 // return the item
 return (ret) ;
} // search and remove a specific element


DoubleNode *
DoublyLinkedImpl::search_for_elem(void  *item)
{
 DoubleNode *n ;

 // loop thru and search for the item
 for ( n= first ; (n != NULL) ; n = n->get_next())
  {

   if ( n->get_elem() == item )
      return ( n ) ;
  } // look for the appropriate node

 // if we get this far - obviously the item was not found
 return (NULL) ;
} // search for a specific element

