/*
//	ML1.cc -- ML1$@%U%)!<%^%C%H2hA|$r07$&(J
//
//		created    in 3/21/1993
//		revised    in 2/14/1993
*/

#include	<stdio.h>
#include	<stdlib.h>
#include	"defs.h"
#include	"ML1.h"
#include	"Pic.h"

#define	SetMark(p)	(Mark[p/8] |= BitField[p%8])
#define	FetchMark(p)	(Mark[p/8] & BitField[p%8])

static int 	ML1Mode;
static int16	ML1Scale[] = { 0, 3, 7, 11, 15, 19, 23, 27, 31 };
static int16 	ColorTable[128];

void	ReadML1Header();
int 	readwyle();
void	ML1Expand();
void	ML1ExpandChain( int, int16 );

void	ML1Load()
{
    if ( fp == NULL && (fp = fopen( File, "rb" )) == NULL )
	error ("%s: No such file");
    AllocateBuffer();
    BitBufferLength = 0;
    ReadML1Header();
    PicMarkClear();
    ML1Expand();
    if ( fp != stdin )
	fclose (fp);
    free (BitBuffer);
    free (Mark);
}

void	ReadML1Header()
{
    int 	x0, y0, x1, y1;
    int 	colornum;
    int 	c, r, g, b, i, j;

    PicBitLoad (8);
    if ( PicWord != '1'  )
	error ("%s: Is not ml1 format");
    PicBitLoad (8);
    if ( PicWord != '0'  )
	error ("%s: Is not ml1 format");
    PicBitLoad (8);
    if ( PicWord != '0'  )
	error ("%s: Is not ml1 format");
    PicBitLoad (8);
    if ( PicWord != '\x1a'  )
	error ("%s: Is not ml1 format");
    PicBitLoad (8); /* year - 1900 */
    PicBitLoad (8); /* month */
    PicBitLoad (8); /* day */
    PicBitLoad (8); /* hour */
    PicBitLoad (8); /* minute */
    PicBitLoad (8); /* second */
    PicBitLoad (16);
    x0 = PicWord;
    PicBitLoad (16);
    y0 = PicWord;
    PicBitLoad (16);
    x1 = PicWord;
    PicBitLoad (16);
    y1 = PicWord;
    for ( i = 0; i < 12; i++ ) {
	PicBitLoad (8); /* handle */
    }
    PicBitLoad (16);
    for ( i = 0; i < 16; i++ )
	PicBitLoad (8);
    for ( i = 0; i < 16; i++ )
	PicBitLoad (8);
    for ( i = 0; i < 32; i++ )
	PicBitLoad (8);
    Width = x1 - x0 + 1;
    Height = y1 - y0 + 1;
    AllocateScreen();
    PicRadix = (Width + 7) / 8;
    if ( (Mark = (unsigned char*)malloc (PicRadix * Height)) == NULL )
	error ("%s: Insufficient memory");
    PicBitLoad (2);
    switch (ML1Mode = PicWord) {
        case 0:
	    colornum = 128;
	    for ( i = 0; i < colornum; i++ ) {
		PicBitLoad (10);
		c = PicWord;
		r = c / 81;
		g = (c - r * 81) / 9;
		b = (c - r * 81 - g * 9);
		ColorTable[i] = (ML1Scale[g] << 10) | (ML1Scale[r] << 5) | ML1Scale[b];
	    }
	    break;
	case 1:
	    PicBitLoad (7);
	    colornum = PicWord + 1;
	    for ( i = 0; i < colornum; i++ ) {
		PicBitLoad (7);
		j = PicWord;
		PicBitLoad (10);
		c = PicWord;
		r = c / 81;
		g = (c - r * 81) / 9;
		b = (c - r * 81 - g * 9);
		ColorTable[i] = (ML1Scale[g] << 10) | (ML1Scale[r] << 5) | ML1Scale[b];
	    }
	    break;
	case 2:
	    PicBitLoad (7);
	    colornum = PicWord + 1;
	    for ( i = 0; i < colornum; i++ ) {
		PicBitLoad (7);
		j = PicWord;
		PicBitLoad (10);
		c = PicWord;
		r = c / 81;
		g = (c - r * 81) / 9;
		b = (c - r * 81 - g * 9);
		ColorTable[i] = (ML1Scale[g] << 10) | (ML1Scale[r] << 5) | ML1Scale[b];
	    }
	    break;
	default:
	    error ("%s: Is not ml1 format");
    }
}

int 	readwyle()
{
    static int 	table[] = { 0, 2, 6, 14, 30, 62, 126, 254, 510, 1022, 2046, 4094, 8190 };
    int 	n, off;

    n = 0;
    for ( ;; n++ ) {
	PicBitLoad (1);
	if ( PicWord == 0 )
	    break;
    }
    PicBitLoad (n + 1);
    off = PicWord;
    return table[n] + off;
}

void	ML1Expand()
{
    int 	len, color, p, i;
    int16	c;
    int 	x, y;

    x = 0;
    y = 0;
    p = 0;
    for ( ;; ) {
	len = readwyle();
	if ( len == 16000 )
	    break;
	if ( ML1Mode == 2 ) {
	    color = readwyle();
	}
	else {
	    PicBitLoad (7);
	    color = PicWord;
	}
	c = ColorTable[color];
	PicBitLoad (1);
	if ( PicWord != 0 )
	    ML1ExpandChain( p, c );
	for ( i = 0; i <= len; i++ ) {
	    if ( FetchMark (p) ) {
		c = Screen[p];
	    }
	    Screen[p] = c;
	    if ( ++x >= 160 ) {
		x = 0;
		y++;
		p = y * Width;
	    }
	    else
		p++;
	}
    }
}

void	ML1ExpandChain( int p, int16 c )
{
    for ( ;; ) {
	PicBitLoad (1);
	if ( PicWord == 0 ) {
	    p += Width;
	    Screen[p] = c;
	    SetMark (p);
	    continue;
	}
	PicBitLoad (2);
	switch (PicWord) {
	    case 0:
	        p += Width + 1;
		Screen[p] = c;
		SetMark (p);
		break;
	    case 1:
		p += Width - 1;
		Screen[p] = c;
		SetMark (p);
		break;
	    case 2:
		return;
	    case 3:
		PicBitLoad (1);
		if ( PicWord == 0 ) {
		    p += Width + 2;
		    Screen[p] = c;
		    SetMark (p);
		}
		else {
		    p += Width - 2;
		    Screen[p] = c;
		    SetMark (p);
		}
		break;
	}
    }
}
