/*
//	Pic.h -- pic$@%U%)!<%^%C%H2hA|$r07$&(J
//
//		created    in 3/1/1993
//		revised    in 2/13/1994
*/

#include	<stdio.h>
#include	<stdlib.h>
#include	"defs.h"
#include	"Pic.h"

#define	SetMark(x,y)	(Mark[(x)/8 + (y) * PicRadix] |= BitField[(x)%8])
#define	ResetMark(x,y)	(Mark[(x)/8 + (y) * PicRadix] &= ~BitField[(x)%8])
#define	FetchMark(x,y)	(Mark[(x)/8 + (y) * PicRadix] & BitField[(x)%8])

struct	{
    int16 	Color;
    int 	Next;
    int 	Previous;
}	LRUCache[128];

unsigned char	*BitBufferPointer;
int32	PicRadix;
unsigned char	*Mark;
uint32 	PicWord;
int 	NoExpand;
int 	ColorPointer;

void	ColorCachInitialize();
void	ReadPicHeader();
void	Expand();
int16	ReadColor();
int16 	NewColor( int16 c );
int16	GetColor( int i );
uint32	ReadLength();
void	ExpandChain( int x, int y, int c );
void	WritePicHeader();
void	DifferentPointMark();
void	CompressPic();
void	CompressChain( int x, int y, int c );
void	WriteLength( uint32 Length );
void	WriteColor( int16 c );
int 	SearchColor( int16 c );
void	SetColor( int i );

void	PicLoad()
{
    int 	OldWidth, x, y;

    if ( fp == NULL && (fp = fopen( File, "rb" )) == NULL )
	error ("%s: No such file");
    AllocateBuffer();
    BitBufferLength = 0;
    ColorCachInitialize();
    ReadPicHeader();
    if ( (Mark = (unsigned char*)malloc (PicRadix * Height)) == NULL )
	error ("%s: Insufficient memory");
    Size = Width * Height * 2;
    PicMarkClear();
    Expand();
    free (Mark);
    free (BitBuffer);
    if ( fp != stdin )
	fclose (fp);
    if ( !NoExpand ) {
	OldWidth = Width;
	Width = Width * 4 / 3;
	Size = Width * Height * 2;
	for ( y = Height - 1; y >= 0; y-- ) {
	    for ( x = Width - 1; x >= 0; x-- ) {
		Screen[x + y * Width] = Screen[x * 3 / 4 + y * OldWidth];
	    }
	}
    }
    MaxColorGuaranteed = 32768;
}

void	ColorCachInitialize()
{
    int 	i;

    for ( i = 0; i < 128; i++ ) {
	LRUCache[i].Color = 0;
	LRUCache[i].Previous = i + 1;
	LRUCache[i].Next = i - 1;
    }
    LRUCache[127].Previous = 0;
    LRUCache[0].Next = 127;
    ColorPointer = 0;
}

void	ReadPicHeader()
{
    int 	i;

    PicBitLoad (8);
    if ( PicWord != 'P' )
	error ("%s: Is not pic format");
    PicBitLoad (8);
    if ( PicWord != 'I' )
	error ("%s: Is not pic format");
    PicBitLoad (8);
    if ( PicWord != 'C' )
	error ("%s: Is not pic format");
    for ( i = 0; i < 16384; i++ ) {
	PicBitLoad (8);
	if ( PicWord == 26 )
	    break;
    }
    if ( i == 16384 )
	error ("%s: Is not pic format");
    for ( i = 0; i < 16384; i++ ) {
	PicBitLoad (8);
	if ( PicWord == 0 )
	    break;
    }
    if ( i == 16384 )
	error ("%s: Is not pic format");
    PicBitLoad (16);
    if ( PicWord != 0 )
	error ("%s: Is not pic format");
    PicBitLoad (16);
    if ( PicWord != 15 )
	error ("%s: Is not pic format");
    PicBitLoad (16);
    Width = PicWord;
    PicBitLoad (16);
    Height = PicWord;
    PicRadix = (Width + 7) / 8;
    if ( Screen != NULL )
	free (Screen);
    if ( (Screen = (int16*)malloc (4 * Width * Height * sizeof (int16) / 3)) == NULL )
	error ("%s: Insufficient memory");
}

void	PicMarkClear()
{
    int 	i;

    for ( i = 0; i < PicRadix * Height; i++ )
	Mark[i] = 0;
}

void	PicBufferNext()
{
    if ( BitBufferLength == 0 ) {
	if ( (BitBufferLength = fread( BitBuffer, 1, BitBufferSize, fp )) == 0 )
	    error ("%s: Unexpected EOF");
	BitBufferPointer = BitBuffer;
    }
    else
	BitBufferPointer++;
    BitBufferLength--;
    BitLength = 8;
}

void	Expand()
{
    int16 	c, a;
    uint32	Length;
    int 	x, y;

    x = -1;
    y = c = 0;
    for ( ;; ) {
	Length = ReadLength();
	while (--Length) {
	    if ( ++x == Width ) {
		if ( ++y == Height )
		    return;
		x = 0;
	    }
	    if ( FetchMark( x, y ) != 0 ) {
		a = Screen[x + y * Width];
		c = a;
		ResetMark( x, y );
	    }
	    Screen[x + y * Width] = c;
	}
	if ( ++x == Width ) {
	    if ( ++y == Height )
		return;
	    x = 0;
	}
	c = ReadColor();
	Screen[x + y * Width] = c;
	PicBitLoad (1);
	if ( PicWord != 0 ) {
	    ExpandChain( x, y, c );
	}
    }
}

int16	ReadColor()
{
    PicBitLoad (1);
    if ( PicWord == 0 ) {
	PicBitLoad (15);
	return NewColor (PicWord);
    }
    else {
	PicBitLoad (7);
	return GetColor (PicWord);
    }
}

int16 	NewColor( int16 c )
{
    ColorPointer = LRUCache[ColorPointer].Previous;
    LRUCache[ColorPointer].Color = c;
    return c;
}

int16	GetColor( int i )
{
    if ( ColorPointer != i ) {
	LRUCache[LRUCache[i].Previous].Next = LRUCache[i].Next;
	LRUCache[LRUCache[i].Next].Previous = LRUCache[i].Previous;
	LRUCache[LRUCache[ColorPointer].Previous].Next = i;
	LRUCache[i].Previous = LRUCache[ColorPointer].Previous;
	LRUCache[ColorPointer].Previous = i;
	LRUCache[i].Next = ColorPointer;
	ColorPointer = i;
    }
    return LRUCache[i].Color;
}

uint32	ReadLength()
{
    int 	a = 1;

    for ( ;; ) {
	PicBitLoad (1);
	if ( PicWord == 0 )
	    break;
	a++;
    }
    PicBitLoad (a);
    return (uint32)(PicWord + (1 << a) - 1);
}

void	ExpandChain( int x, int y, int c )
{
    int 	yover = 1;

    for ( ;; ) {
	PicBitLoad (2);
	switch (PicWord) {
	    case 0:
	        PicBitLoad (1);
	        if ( PicWord == 0 ) {
		    return;
		}
		PicBitLoad (1);
		if ( PicWord == 0 )
		    x -= 2;
		else
		    x += 2;
		break;
	    case 1:
		x--;
		break;
	    case 2:
		break;
	    case 3:
		x++;
		break;
	}
	if ( x >= Width )
	    return;
	if ( ++y >= Height )
	    yover = 0;
	if ( yover ) {
	    SetMark( x, y );
	    Screen[x + y * Width] = c;
	}
    }
}

void	PicSave()
{
    int 	OldWidth, x, y;

    if ( fp == NULL && (fp = fopen( File, "wb" )) == NULL )
	error ("%s: Cannot create");
    AllocateBufferForWriting();
    BitBufferLength = BitBufferSize;
    BitLength = 8;
    BitBufferPointer = BitBuffer;
    ColorCachInitialize();
    if ( !NoExpand ) {
	OldWidth = Width;
	Width = Width * 3 / 4;
	for ( y = 0; y < Height; y++ ) {
	    for ( x = 0; x < OldWidth; x++ ) {
		Screen[x * 3 / 4 + y * Width] = Screen[x + y * OldWidth];
	    }
	}
    }
    WritePicHeader();
    PicRadix = (Width + 7) / 8;
    if ( (Mark = (unsigned char*)malloc (PicRadix * Height)) == NULL )
	error ("%s: Insufficient memory");
    PicMarkClear();
    DifferentPointMark();
    CompressPic();
    PicBufferWriteFlush();
    free (Mark);
    free (BitBuffer);
    if ( fp != stdout )
	fclose (fp);
}

void	WritePicHeader()
{
    PicBitWrite( 8, (uint32)'P' );
    PicBitWrite( 8, (uint32)'I' );
    PicBitWrite( 8, (uint32)'C' );
    PicBitWrite( 8, (uint32)26 );
    PicBitWrite( 8, (uint32)0 );
    PicBitWrite( 16, (uint32)0 );
    PicBitWrite( 16, (uint32)15 );
    PicBitWrite( 16, (uint32)Width );
    PicBitWrite( 16, (uint32)Height );
}

void	DifferentPointMark()
{
    int 	x, y, c, a;

    c = 0;
    for ( y = 0; y < Height; y++ ) {
	for ( x = 0; x < Width; x++ ) {
	    a = Screen[x + y * Width];
	    if ( a != c ) {
		c = a;
		SetMark( x, y );
	    }
	    else
		ResetMark( x, y );
	}
    }
}

void	CompressPic()
{
    uint32	Length;
    int 	x, y;
    int16	a;

    Length = 0;
    for ( y = 0; y < Height; y++ ) {
	for ( x = 0; x < Width; x++ ) {
	    Length++;
	    if ( FetchMark( x, y ) != 0 ) {
		a = Screen[x + y * Width];
		WriteLength (Length);
		WriteColor (a);
		CompressChain( x, y, a );
		ResetMark( x, y );
		Length = 0;
	    }
	}
    }
    WriteLength (Length + 1);
}

void	CompressChain( int x, int y, int c )
{
    int 	yy, d, f;

    f = 0;
    for ( yy = y + 1; yy < Height; yy++ ) {
	if ( Screen[x + yy * Width] == c )
	    d = 2;
	else if ( --x >= 0     && Screen[x + yy * Width] == c )
	    d = 1;
	else if ( (x+=2) < Width && Screen[x + yy * Width] == c )
	    d = 3;
	else if ( (x-=3) >= 0  && Screen[x + yy * Width] == c )
	    d = 4;
	else if ( (x+=4) < Width && Screen[x + yy * Width] == c )
	    d = 5;
	else
	    break;
	ResetMark( x, yy );
	if ( f == 0 )
	    PicBitWrite( 1, (uint32)1 );
	if ( d == 4 )
	    PicBitWrite( 4, (uint32)2 );
	if ( d == 5 )
	    PicBitWrite( 4, (uint32)3 );
	if ( d <= 3 )
	    PicBitWrite( 2, (uint32)d );
	f = 1;
    }
    if ( f == 0 ) {
	PicBitWrite( 1, (uint32)0 );
    }
    else {
	PicBitWrite( 3, (uint32)0 );
    }
}

void	WriteLength( uint32 Length )
{
    int 	a;
    uint32	b;

    a = 1;
    b = 4;
    while ( Length > b - 2 ) {
	a++;
	b = b * 2;
    }
    PicBitWrite( a, 0xfffffffe );
    PicBitWrite( a, (uint32)(Length + 1 - b / 2) );
}

void	WriteColor( int16 c )
{
    int 	a;

    if ( (a = SearchColor (c)) != -1 ) {
	PicBitWrite( 8, (uint32)(a + 128) );
    }
    else {
	PicBitWrite( 1, 0 );
	PicBitWrite( 15, (uint32)c );
    }
}

int 	SearchColor( int16 c )
{
    int 	i;

    for ( i = 0; i < 128; i++ ) {
	if ( LRUCache[i].Color == c )
	    break;
    }
    if ( i == 128 ) {
	NewColor (c);
	i = -1;
    }
    else
	SetColor( i );
    return (i);
}

void	SetColor( int i )
{
    if ( ColorPointer != i ) {
	LRUCache[LRUCache[i].Previous].Next = LRUCache[i].Next;
	LRUCache[LRUCache[i].Next].Previous = LRUCache[i].Previous;
	LRUCache[LRUCache[ColorPointer].Previous].Next = i;
	LRUCache[i].Previous = LRUCache[ColorPointer].Previous;
	LRUCache[ColorPointer].Previous = i;
	LRUCache[i].Next = ColorPointer;
	ColorPointer = i;
    }
}

void	PicWriteBufferNext()
{
    if ( --BitBufferLength == 0 ) {
	if ( fwrite( BitBuffer, 1, BitBufferSize, fp ) != BitBufferSize )
	    error ("%s: Error in writing");
	BitBufferPointer = BitBuffer;
	BitBufferLength = BitBufferSize;
    }
    else
	++BitBufferPointer;
    BitLength = 8;
}

void	PicBufferWriteFlush()
{
    if ( BitBufferLength > 0 ) {
	*BitBufferPointer <<= BitLength;
	PicWriteBufferNext();
    }
    BitBufferLength = BitBufferSize - BitBufferLength;
    if ( BitBufferLength > 0 ) {
	if ( fwrite( BitBuffer, 1, BitBufferLength, fp ) != BitBufferLength )
	    error ("%s: Error in writing");
    }
}
