/*****************************************************************************
   MODULE: _rcgetf.h
  PURPOSE: recio column delimited floating point input functions
COPYRIGHT: (C) 1994 William Pierpoint
  VERSION: 2.00
  RELEASE: April 15, 1994
*****************************************************************************/

#ifndef _RCGETF_H
#define _RCGETF_H

#include "recio.h"

extern int _rstatus(REC *rp, int mode);
extern char *_rfldstr(REC *rp, size_t len);
extern char *_rerrs(REC *rp, int errnum);

#define rcol(rp) (rp->r_colno)
#define READ 0

/* clip value v between lower l and upper u bounds */
#define range(l, v, u)  (min(max((l),(v)),(u)))

/* macro to get column delimited floating point number */
#define rcget_fn( /* define function to get number from record */\
    fn_type,      /* defined function return type */\
    fn_name,      /* defined function name */\
    fn_err,       /* defined function error return value */\
    cv_type,      /* conversion function return type */\
    cv_name,      /* conversion function name */\
    fn_negmin,    /* inclusive valid negative minimum value */\
    fn_negmax,    /* inclusive valid negative maximum value */\
    fn_posmin,    /* inclusive valid positive minimum value */\
    fn_posmax)    /* inclusive valid positive maximum value */\
\
fn_type fn_name(        /* return fn_type, return fn_err on error */\
        REC *rp,        /* record pointer */\
        size_t begcol,  /* field inclusive beginning column */\
        size_t endcol)  /* field inclusive ending column */\
{ \
    fn_type result=(fn_err); /* result to return */\
    cv_type val;             /* conversion value */\
    char *fldptr;            /* pointer to field string */\
    char *endptr;            /* pointer to first invalid field char */\
    char *fldp;              /* another pointer to field string */\
\
    if (!_rstatus(rp, READ)) { \
      if (endcol >= begcol && begcol >= rbegcolno(rp)) { \
        rcol(rp) = begcol - rbegcolno(rp); \
        fldptr = _rfldstr(rp, endcol-begcol+1); \
        if (fldptr) { \
          for (;;) { \
            for (fldp=fldptr; *fldp; fldp++) {if (!isspace(*fldp)) break;} \
            if (*fldp) { \
              endptr = fldptr; \
              val = cv_name(fldptr, &endptr); \
              while (isspace(*endptr)) endptr++; \
              if (errno==ERANGE || !*endptr) { \
                if (!errno) { \
                  if (!val || (val>=(fn_negmin) && val<=(fn_negmax)) || \
                              (val>=(fn_posmin) && val<=(fn_posmax))) { \
                        result = (fn_type) val; \
                        goto done; \
                  } \
                } /* out of range */ \
                fldptr = _rerrs(rp, R_ERANGE); \
                if (fldptr) { continue; } else { goto done; } \
              } /* invalid data */ \
              fldptr = _rerrs(rp, R_EINVDAT); \
              if (fldptr) { continue; } else { goto done; } \
            } /* missing data (empty fldstr) */ \
            fldptr = _rerrs(rp, R_EMISDAT); \
            if (fldptr) { continue; } else { goto done; } \
          } \
        } /* null pointer */ \
      } /* column args reversed or tried to start before first column */ \
      rseterr(rp, R_EINVAL); \
    } \
done: \
    return result; \
}

#endif
