//*BHEADER* :ts=8  -*- C++ -*-
/*****************************************************************************
 *
 *   |_|_|_  |_|_    |_    |_|_|_  |_		     C O M M U N I C A T I O N
 * |_        |_  |_  |_  |_        |_		               N E T W O R K S
 * |_        |_  |_  |_  |_        |_		                     C L A S S
 *   |_|_|_  |_    |_|_    |_|_|_  |_|_|_|_	                 L I B R A R Y
 *
 * $Id: DLIterator.c,v 0.30 1995/11/22 09:56:23 turm Exp $
 *
 * Class: CNDLIterator --- Double linked list iterator
 *
 *****************************************************************************
 * Copyright (C) 1992-1995   Communication Networks
 *                           Aachen University of Technology
 *                           D-52056 Aachen
 *                           Germany
 *                           Email: cncl-adm@dfv.rwth-aachen.de
 *****************************************************************************
 * This file is part of the CN class library. All files marked with
 * this header are free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.  This library is
 * distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.  You should have received a copy of the GNU
 * Library General Public License along with this library; if not, write
 * to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 * 
 * As an exception to this rule you may use this template to generate
 * your own classes. This does not cause these classes to be covered by
 * the GNU Library General Public License. This exception does not
 * however invalidate any other reasons why the resulting program must be
 * covered by the GNU Library General Public License.
 **EHEADER********************************************************************/

#include <CNCL/Class.h>

#include "DLIterator.h"


CNObject *CNDLIterator::last_object()
{
    pnt = list ? list->last() : NIL;
    return pnt ? pnt->object() : NIL;
}
 
CNObject *CNDLIterator::last()
{
    pnt = list ? list->last() : NIL;
    return pnt ? pnt->object() : NIL;
}

CNObject *CNDLIterator::prev_object()
{
    CNDLObject *p;
    p = CNDLObject::cast_from_object(pnt);
    if(pnt)
	pnt = CNDLObject::cast_from_object(pnt)->pprev;
    return p ? p->object() : NIL;
}

CNObject *CNDLIterator::prev()
{
    return prev_object();
}

CNObject *CNDLIterator::operator--()
{
    return prev_object();
}

CNObject *CNDLIterator::operator--(int)
{
    return prev_object();
}

CNDLObject *CNDLIterator::position()
{
    return CNDLObject::cast_from_object(pnt);
}

CNDLObject *CNDLIterator::get_position()
{
    return CNDLObject::cast_from_object(pnt);
}



/***** Default I/O member function for CNCL classes **************************/

// Normal output
void CNDLIterator::print(ostream &strm) const
{
    strm << "..." << endl;
}

// Debug output
void CNDLIterator::dump(ostream &strm) const
{
    strm << "CNDLIterator { $Revision: 0.30 $ ..."
	 << " }" << endl;
}

// IOStream operator <<
ostream &operator << (ostream &strm, const CNDLIterator &obj)
{
    obj.print(strm);
    return strm;
}

ostream &operator << (ostream &strm, const CNDLIterator *obj)
{
    if(obj)
	obj->print(strm);
    else
	strm << "(NIL)";
    return strm;
}



/***** CNCL stuff for type information ***************************************/

// Describing object for class CNDLIterator
static CNClass CNDLIterator_desc("CNDLIterator", "$Revision: 0.30 $",
			    CNDLIterator::new_object);

// "Type" for type checking functions
CNClassDesc CN_DLITERATOR = &CNDLIterator_desc;
