//*BHEADER* :ts=8  -*- C++ -*-
/*****************************************************************************
 *
 *   |_|_|_  |_|_    |_    |_|_|_  |_		     C O M M U N I C A T I O N
 * |_        |_  |_  |_  |_        |_		               N E T W O R K S
 * |_        |_  |_  |_  |_        |_		                     C L A S S
 *   |_|_|_  |_    |_|_    |_|_|_  |_|_|_|_	                 L I B R A R Y
 *
 * $Id: KeyInt.h,v 0.30 1995/08/10 20:03:03 steppler Exp $
 *
 * CNClass: CNKeyInt --- CNObject management via integer keys
 *
 *****************************************************************************
 * Copyright (C) 1992-1995   Communication Networks
 *                           Aachen University of Technology
 *                           D-52056 Aachen
 *                           Germany
 *                           Email: cncl-adm@dfv.rwth-aachen.de
 *****************************************************************************
 * This file is part of the CN class library. All files marked with
 * this header are free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.  This library is
 * distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.  You should have received a copy of the GNU
 * Library General Public License along with this library; if not, write
 * to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 **EHEADER********************************************************************/

#ifndef __KeyInt_h
#define __KeyInt_h


#include <CNCL/Class.h>
#include <CNCL/Param.h>

#include <CNCL/Key.h>		// Base class

extern CNClassDesc CN_KEYINT;	// CNClass CNKeyInt description

/*
 * The class CNKeyInt
 */

class CNKeyInt : public CNKey
{
    unsigned long ki_CNKeyInt;
    
public:
    unsigned long get_key() const
    {
	return ki_CNKeyInt;
    }

    virtual unsigned long hash(unsigned long capacity, int par = 0) const;
    virtual bool compare(const CNKey *k) const;
    virtual bool compare(const CNKey &k) const
    { return compare(&k); }

    /***** Constructors ******************************************************/
public:
    CNKeyInt() : CNKey((CNObject *)NIL), ki_CNKeyInt(0) {}
    CNKeyInt(unsigned long key_int, CNObject *obj = NIL);
    CNKeyInt(CNParam *);

    /***** Member functions required by CNCL *********************************/
public:
    virtual CNClassDesc class_desc() const	// CNClass description
    {
	return CN_KEYINT;
    };
            
    virtual bool is_a(CNClassDesc desc) const	// Type checking
    {
	return desc == CN_KEYINT ? TRUE : CNKey::is_a(desc);
    };
        
    static CNKeyInt *cast_from_object(CNObject *obj) // Safe type cast
    {
#   ifdef NO_TYPE_CHECK
	return (CNKeyInt *)obj;
#   else
	return (CNKeyInt *)( !obj || obj->is_a(CN_KEYINT)
	       ? obj : fatal_type(obj->class_desc(), CN_KEYINT) );
#   endif
    }
    
    static CNObject *new_object(CNParam *param = NIL) // Object creation
    { return param ? new CNKeyInt(param) : new CNKeyInt; }
    
    // Print/debug output
    virtual void print(ostream &strm = cout) const;
    virtual void dump (ostream &strm = cout) const;
};


#endif /**__KeyInt_h**/
