//*BHEADER* :ts=8  -*- C++ -*-
/*****************************************************************************
 *
 *   |_|_|_  |_|_    |_    |_|_|_  |_		     C O M M U N I C A T I O N
 * |_        |_  |_  |_  |_        |_		               N E T W O R K S
 * |_        |_  |_  |_  |_        |_		                     C L A S S
 *   |_|_|_  |_    |_|_    |_|_|_  |_|_|_|_	                 L I B R A R Y
 *
 * $Id: Stack.c,v 0.29 1995/07/05 12:47:20 cncl-adm Exp $
 *
 * Class: CNStack --- ...
 *
 *****************************************************************************
 * Copyright (C) 1992-1995   Communication Networks
 *                           Aachen University of Technology
 *                           D-52056 Aachen
 *                           Germany
 *                           Email: cncl-adm@dfv.rwth-aachen.de
 *****************************************************************************
 * This file is part of the CN class library. All files marked with
 * this header are free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.  This library is
 * distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.  You should have received a copy of the GNU
 * Library General Public License along with this library; if not, write
 * to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 * 
 * As an exception to this rule you may use this template to generate
 * your own classes. This does not cause these classes to be covered by
 * the GNU Library General Public License. This exception does not
 * however invalidate any other reasons why the resulting program must be
 * covered by the GNU Library General Public License.
 **EHEADER********************************************************************/

#include "Stack.h"



void CNStack::push(CNObject *obj)
{
    if (!s_size || size()-depth())
    {
	queue.put(obj);
	s_depth++;
    }
    else
    {
	fatal("CNStack::push()"," stack overflow");
    }
}



CNObject *CNStack::pull()
{
    CNObject *obj = NIL;
    if (depth())
    {
	obj = queue.get();
	s_depth--;
    }
    else
    {
	fatal("CNStack::pull()"," stack empty");
    }
    return obj;
}	



void CNStack::size(long len)
{
    if (len >= depth())
	s_size = len;
    else
	warning("CNStack::size()"," cannot resize stack below current depth");
}



void CNStack::clear()
{
    queue.delete_all();
}



/***** Default I/O member function for CNCL classes **************************/

// Normal output
void CNStack::print(ostream &strm) const
{
    strm << "size=" << s_size << " depth=" << s_depth << endl;
}

// Debug output
void CNStack::dump(ostream &strm) const
{
    strm << "CNStack { $Revision: 0.29 $ ..." << endl
         << "size=" << s_size << " depth=" << s_depth << endl;
    strm << " }" << endl;
    queue.dump();
}



/***** CNCL stuff for type information ***************************************/

// Describing object for class CNStack
static CNClass CNStack_desc("CNStack", "$Revision: 0.29 $",
			    CNStack::new_object);

// "Type" for type checking functions
CNClassDesc CN_STACK = &CNStack_desc;
