//*BHEADER* :ts=8  -*- C++ -*-
/*****************************************************************************
 *
 *   |_|_|_  |_|_    |_    |_|_|_  |_		     C O M M U N I C A T I O N
 * |_        |_  |_  |_  |_        |_		               N E T W O R K S
 * |_        |_  |_  |_  |_        |_		                     C L A S S
 *   |_|_|_  |_    |_|_    |_|_|_  |_|_|_|_	                 L I B R A R Y
 *
 * $Id: EventExploder.c,v 0.29 1995/07/05 12:48:04 cncl-adm Exp $
 *
 * Class: CNEventExploder --- send CNEvents to many CNEventHandlers
 *
 *****************************************************************************
 * Copyright (C) 1992-1995   Communication Networks
 *                           Aachen University of Technology
 *                           D-52056 Aachen
 *                           Germany
 *                           Email: cncl-adm@dfv.rwth-aachen.de
 *****************************************************************************
 * This file is part of the CN class library. All files marked with
 * this header are free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.  This library is
 * distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.  You should have received a copy of the GNU
 * Library General Public License along with this library; if not, write
 * to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 **EHEADER********************************************************************/

#include "EventExploder.h"
#include <CNCL/DLObject.h>


void CNEventExploder::add_handler(CNEventHandler *eh) {
    handlers.append(eh);
};

void CNEventExploder::rem_handler(CNEventHandler *eh) {
    CNDLObject *n = handlers.first();
    while (n) {
	if (n->object() == eh) {
	    handlers.delete_object(n);
	    return;
	};
	n = n->next();
    };
};

/***
void CNEventExploder::process_event(CNEventScheduler *sched,
				   const CNSimTime t, CNEvent *ev) {
    CNDLObject *n = handlers.first();
    while (n) {
	CNCAST(CNEventHandler,n->object())->process_event(sched,t,ev);
	n = n->next();
    };
};
***/

void CNEventExploder::event_handler(const CNEvent *ev) {
    CNDLObject *n = handlers.first();
    while (n) {
	CNCAST(CNEventHandler,n->object())
	  ->process_event(ev_scheduler,ev_now,ev);
	n = n->next();
    };
};


/***** Default I/O member function for CNCL classes **************************/

// Normal output
void CNEventExploder::print(ostream &strm) const
{
    handlers.print(strm);
}

// Debug output
void CNEventExploder::dump(ostream &strm) const
{
    strm << "CNEventExploder { $Revision: 0.29 $ " << endl;
    handlers.dump(strm);
    strm << " }" << endl;
}



/***** CNCL stuff for type information ***************************************/

// Describing object for class CNEventExploder
static CNClass CNEventExploder_desc("CNEventExploder", "$Revision: 0.29 $",
			    CNEventExploder::new_object);

// "Type" for type checking functions
CNClassDesc CN_EVENTEXPLODER = &CNEventExploder_desc;
