//*BHEADER* :ts=8  -*- C++ -*-
/*****************************************************************************
 *
 *   |_|_|_  |_|_    |_    |_|_|_  |_		     C O M M U N I C A T I O N
 * |_        |_  |_  |_  |_        |_		               N E T W O R K S
 * |_        |_  |_  |_  |_        |_		                     C L A S S
 *   |_|_|_  |_    |_|_    |_|_|_  |_|_|_|_	                 L I B R A R Y
 *
 * $Id: Int.h,v 0.29 1995/07/05 12:47:54 cncl-adm Exp $
 *
 * Class: CNInt --- ...
 *
 *****************************************************************************
 * Copyright (C) 1992-1995   Communication Networks
 *                           Aachen University of Technology
 *                           D-52056 Aachen
 *                           Germany
 *                           Email: cncl-adm@dfv.rwth-aachen.de
 *****************************************************************************
 * This file is part of the CN class library. All files marked with
 * this header are free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.  This library is
 * distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.  You should have received a copy of the GNU
 * Library General Public License along with this library; if not, write
 * to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 **EHEADER********************************************************************/

#ifndef __CNInt_h
#define __CNInt_h


#include <CNCL/Class.h>
#include <CNCL/Param.h>

#include <CNCL/Object.h>		// Base class

extern CNClassDesc CN_INT;	// Class CNInt description


/*
 * The class CNInt
 */

class CNInt : public CNObject
{
  public:	/***** Constructors ******************************************/
    CNInt(long n=0):value(n) {}			// Default constructor
    CNInt(CNParam *) {}			// CNParam constructor

  public:	/***** Public interface **************************************/
    operator long() { return value; }

    long operator ++()    { return ++value; }
    long operator ++(int) { return value++; } 
    long operator --()    { return --value; }
    long operator --(int) { return value--; }
    
    long operator -() { return value = -value; }
    long operator +() { return value; }
 
    long operator +=(long n) { return value += n; }
    long operator -=(long n) { return value -= n; }
    long operator *=(long n) { return value *= n; }
    long operator /=(long n) { return value /= n; }
    long operator %=(long n) { return value %= n; }

    long operator ^=(long n) { return value ^= n; }
    long operator |=(long n) { return value |= n; }
    long operator &=(long n) { return value &= n; }
    long operator !=(long n) { return value != n; }

    long operator <<=(int n) { return value <<= n; }
    long operator >>=(int n) { return value >>= n; }
   
  protected:	/***** Internal private members ******************************/
    long value;        // that's what all is about


  public:	/***** Member functions required by CNCL *********************/
    virtual CNClassDesc class_desc() const	// CNClass description
    { return CN_INT; }
            
    virtual bool is_a(CNClassDesc desc) const	// Type checking
    { return desc == CN_INT ? TRUE : CNObject::is_a(desc); }
        
    static CNInt *cast_from_object(CNObject *obj) // Safe type cast
    {
#   ifdef NO_TYPE_CHECK
	return (CNInt *)obj;
#   else
	return (CNInt *)( !obj || obj->is_a(CN_INT)
	       ? obj : fatal_type(obj->class_desc(), CN_INT) );
#   endif
    }
    
    static CNObject *new_object(CNParam *param = NIL) // Object creation
    { return param ? new CNInt(param) : new CNInt; }
    
    // Print/debug output
    virtual void print(ostream &strm = cout) const;
    virtual void dump (ostream &strm = cout) const;
};

#endif /**__CNInt_h**/
