//*BHEADER* :ts=8  -*- C++ -*-
/*****************************************************************************
 *
 *   |_|_|_  |_|_    |_    |_|_|_  |_		     C O M M U N I C A T I O N
 * |_        |_  |_  |_  |_        |_		               N E T W O R K S
 * |_        |_  |_  |_  |_        |_		                     C L A S S
 *   |_|_|_  |_    |_|_    |_|_|_  |_|_|_|_	                 L I B R A R Y
 *
 * $Id: Erlang.c,v 0.29 1995/07/05 12:47:34 cncl-adm Exp $
 *
 * CNClass: CNErlang --- CNErlang-k distributed random numbers
 *
 *****************************************************************************
 * Copyright (C) 1992-1995   Communication Networks
 *                           Aachen University of Technology
 *                           D-52056 Aachen
 *                           Germany
 *                           Email: cncl-adm@dfv.rwth-aachen.de
 *****************************************************************************
 * This file is part of the CN class library. All files marked with
 * this header are free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.  This library is
 * distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.  You should have received a copy of the GNU
 * Library General Public License along with this library; if not, write
 * to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *****************************************************************************
 * original Copyright:
 * -------------------
 * Copyright (C) 1988 Free Software Foundation
 *    written by Dirk Grunwald (grunwald@cs.uiuc.edu)
 * 
 * This file is part of the GNU C++ Library.  This library is free
 * software; you can redistribute it and/or modify it under the terms of
 * the GNU Library General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  This library is distributed in the hope
 * that it will be useful, but WITHOUT ANY WARRANTY; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU Library General Public License for more details.
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 **EHEADER********************************************************************/

#include "Erlang.h"



void CNErlang::setState()
{
    k = int( (pMean * pMean ) / pVariance + 0.5 );
    k = (k > 0) ? k : 1;
    a = k / pMean;
}

CNErlang::CNErlang(double mean, double variance, CNRNG *gen) : CNRandom(gen)
{
    pMean = mean; pVariance = variance;
    assert( ( pMean > 0 ) && ( pVariance > 0) ); // this line added
    setState();
}

CNErlang::CNErlang() 
	 : CNRandom((CNRNG*)NIL)
{
    pMean = 0;
    pVariance = 0;
    a = 0; 
    k = 0;
    
}

CNErlang::CNErlang(CNParam *) 
	  : CNRandom ((CNRNG*)NIL)
{
    pMean = 0;
    pVariance = 0;
    a = 0;
    k = 0;
}    
    
double CNErlang::mean() { return pMean; }
double CNErlang::mean(double x) {
  double tmp = pMean; 
  pMean = x; 
  setState(); 
  return tmp;
};

double CNErlang::variance() { return pVariance; }
double CNErlang::variance(double x) {
  double tmp = pVariance; 
  pVariance = x; 
  setState(); 
  return tmp;
}



double CNErlang::operator()()
{
    double prod = 1.0;

    for (int i = 0; i < k; i++) {
	prod *= pGenerator -> as_double();
    }
    return(-log(prod)/a);
}



/***** Default I/O member function for CNCL classes **************************/

// CNNormal output
void CNErlang::print(ostream &strm) const
{
    strm << "mean=" << pMean << " var=" << pVariance
	 << " k=" << k << " a=" << a << endl;
}

// Debug output
void CNErlang::dump(ostream &strm) const
{
    strm << "CNErlang { "
	 << "mean=" << pMean << " var=" << pVariance
	 << " k=" << k << " a=" << a
	 << " }" << endl;
}



/***** CNCL stuff for type information and exemplar objects ******************/

// Describing object for class CNErlang
static CNClass CNErlang_desc("CNErlang", "$Revision: 0.29 $", CNErlang::new_object);

// "Type" for type checking functions
CNClassDesc CN_ERLANG = &CNErlang_desc;






