//*BHEADER* :ts=8  -*- C++ -*-
/*****************************************************************************
 *
 *   |_|_|_  |_|_    |_    |_|_|_  |_		     C O M M U N I C A T I O N
 * |_        |_  |_  |_  |_        |_		               N E T W O R K S
 * |_        |_  |_  |_  |_        |_		                     C L A S S
 *   |_|_|_  |_    |_|_    |_|_|_  |_|_|_|_	                 L I B R A R Y
 *
 * $Id: Geometric.c,v 0.29 1995/07/05 12:47:34 cncl-adm Exp $
 *
 * CNClass: CNGeometric --- CNGeometric distributed random numbers
 *
 *****************************************************************************
 * Copyright (C) 1992-1995   Communication Networks
 *                           Aachen University of Technology
 *                           D-52056 Aachen
 *                           Germany
 *                           Email: cncl-adm@dfv.rwth-aachen.de
 *****************************************************************************
 * This file is part of the CN class library. All files marked with
 * this header are free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.  This library is
 * distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.  You should have received a copy of the GNU
 * Library General Public License along with this library; if not, write
 * to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *****************************************************************************
 * original Copyright:
 * -------------------
 * Copyright (C) 1988 Free Software Foundation
 *    written by Dirk Grunwald (grunwald@cs.uiuc.edu)
 * 
 * This file is part of the GNU C++ Library.  This library is free
 * software; you can redistribute it and/or modify it under the terms of
 * the GNU Library General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  This library is distributed in the hope
 * that it will be useful, but WITHOUT ANY WARRANTY; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU Library General Public License for more details.
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 **EHEADER********************************************************************/

#include "Geometric.h"



CNGeometric::CNGeometric(double mean, CNRNG *gen) : CNRandom(gen)
{
  pMean = mean;
}

CNGeometric::CNGeometric() : CNRandom((CNRNG*)NIL)
{
    pMean = 0;
}

CNGeometric::CNGeometric(CNParam *) : CNRandom((CNRNG*)NIL)
{
    pMean = 0;
}


double CNGeometric::mean() { return pMean; }
double CNGeometric::mean(double x) {
  double tmp = pMean;
  pMean = x; 
  return tmp;
}



double CNGeometric::operator()()
{
    int samples;
    for (samples = 1; pGenerator -> as_double() < pMean; samples++);
    return((double) samples);
}



/***** Default I/O member function for CNCL classes **************************/

// CNNormal output
void CNGeometric::print(ostream &strm) const
{
    strm << "p=" << pMean << endl;
}

// Debug output
void CNGeometric::dump(ostream &strm) const
{
    strm << "CNGeometric { "
	 << "p=" << pMean
	 << " }" << endl;
}



/***** CNCL stuff for type information and exemplar objects ******************/

// Describing object for class CNGeometric
static CNClass CNGeometric_desc("CNGeometric", "$Revision: 0.29 $",
			    CNGeometric::new_object);

// "Type" for type checking functions
CNClassDesc CN_GEOMETRIC = &CNGeometric_desc;






