//*BHEADER* :ts=8  -*- C++ -*-
/*****************************************************************************
 *
 *   |_|_|_  |_|_    |_    |_|_|_  |_		     C O M M U N I C A T I O N
 * |_        |_  |_  |_  |_        |_		               N E T W O R K S
 * |_        |_  |_  |_  |_        |_		                     C L A S S
 *   |_|_|_  |_    |_|_    |_|_|_  |_|_|_|_	                 L I B R A R Y
 *
 * $Id: Normal.c,v 0.29 1995/07/05 12:47:34 cncl-adm Exp $
 *
 * CNClass: CNNormal --- CNNormal (Gaussian) distributed random numbers
 *
 *****************************************************************************
 * Copyright (C) 1992-1995   Communication Networks
 *                           Aachen University of Technology
 *                           D-52056 Aachen
 *                           Germany
 *                           Email: cncl-adm@dfv.rwth-aachen.de
 *****************************************************************************
 * This file is part of the CN class library. All files marked with
 * this header are free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.  This library is
 * distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.  You should have received a copy of the GNU
 * Library General Public License along with this library; if not, write
 * to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *****************************************************************************
 * original Copyright:
 * -------------------
 * Copyright (C) 1988 Free Software Foundation
 *    written by Dirk Grunwald (grunwald@cs.uiuc.edu)
 * 
 * This file is part of the GNU C++ Library.  This library is free
 * software; you can redistribute it and/or modify it under the terms of
 * the GNU Library General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  This library is distributed in the hope
 * that it will be useful, but WITHOUT ANY WARRANTY; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU Library General Public License for more details.
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 **EHEADER********************************************************************/

#include "Normal.h"




CNNormal::CNNormal()
	 : CNRandom((CNRNG*)NIL)
{
    pMean = 0;
    pVariance = 0;
    pStdDev = 0;
    haveCachedCNNormal = 0;
}
    


CNNormal::CNNormal(CNParam *)
	 : CNRandom((CNRNG*)NIL)
{
    pMean = 0;
    pVariance = 0;
    pStdDev = 0;
    haveCachedCNNormal = 0;
}



CNNormal::CNNormal(double xmean, double xvariance, CNRNG *gen)
    : CNRandom(gen)
{
    pMean = xmean;
    pVariance = xvariance;
    pStdDev = sqrt(pVariance);
    haveCachedCNNormal = 0;
}



double CNNormal::mean() { return pMean; };
double CNNormal::mean(double x) {
    double t=pMean; 
    pMean = x;
    return t;
}

double CNNormal::variance() { return pVariance; }
double CNNormal::variance(double x) {
    double t = pVariance; 
    pVariance = x;
    pStdDev = sqrt(pVariance);
    return t;
};



//
//  See Simulation, Modelling & Analysis by Law & Kelton, pp259
//
//  This is the ``polar'' method.
// 

double CNNormal::operator()()
{
    if (haveCachedCNNormal == 1) {
	haveCachedCNNormal = 0;
	return(cachedCNNormal * pStdDev + pMean );
    } else {
    
	for(;;) {
	    double u1 = pGenerator -> as_double();
	    double u2 = pGenerator -> as_double();
	    double v1 = 2 * u1 - 1;
	    double v2 = 2 * u2 - 1;
	    double w = (v1 * v1) + (v2 * v2);
	    
//
//  We actually generate two IID normal distribution variables.
//  We cache the one & return the other.
// 
	    if (w <= 1) {
		double y = sqrt( (-2 * log(w)) / w);
		double x1 = v1 * y;
		double x2 = v2 * y;
		
		haveCachedCNNormal = 1;
		cachedCNNormal = x2;
		return(x1 * pStdDev + pMean);
	    }
	}
    }
}




/***** Default I/O member function for CNCL classes **************************/

// CNNormal output
void CNNormal::print(ostream &strm) const
{
    strm << "mean = " << pMean << " var = " << pVariance
	 << " stddev = " << pStdDev << endl;
}

// Debug output
void CNNormal::dump(ostream &strm) const
{
    strm << "CNNormal { "
	 << "mean = " << pMean << " var = " << pVariance
	 << " stddev = " << pStdDev
	 << " }" << endl;
}



/***** CNCL stuff for type information and exemplar objects ******************/

// Describing object for class CNNormal
static CNClass CNNormal_desc("CNNormal", "$Revision: 0.29 $", CNNormal::new_object);

// "Type" for type checking functions
CNClassDesc CN_NORMAL = &CNNormal_desc;






