//*BHEADER* :ts=8  -*- C++ -*-
/*****************************************************************************
 *
 *   |_|_|_  |_|_    |_    |_|_|_  |_		     C O M M U N I C A T I O N
 * |_        |_  |_  |_  |_        |_		               N E T W O R K S
 * |_        |_  |_  |_  |_        |_		                     C L A S S
 *   |_|_|_  |_    |_|_    |_|_|_  |_|_|_|_	                 L I B R A R Y
 *
 * $Id: Poisson.c,v 0.29 1995/07/05 12:47:34 cncl-adm Exp $
 *
 * CNClass: CNPoisson --- CNPoisson distributed random numbers 
 *
 *****************************************************************************
 * Copyright (C) 1992-1995   Communication Networks
 *                           Aachen University of Technology
 *                           D-52056 Aachen
 *                           Germany
 *                           Email: cncl-adm@dfv.rwth-aachen.de
 *****************************************************************************
 * This file is part of the CN class library. All files marked with
 * this header are free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.  This library is
 * distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.  You should have received a copy of the GNU
 * Library General Public License along with this library; if not, write
 * to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *****************************************************************************
 * original Copyright:
 * -------------------
 * Copyright (C) 1988 Free Software Foundation
 *    written by Dirk Grunwald (grunwald@cs.uiuc.edu)
 * 
 * This file is part of the GNU C++ Library.  This library is free
 * software; you can redistribute it and/or modify it under the terms of
 * the GNU Library General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  This library is distributed in the hope
 * that it will be useful, but WITHOUT ANY WARRANTY; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU Library General Public License for more details.
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 **EHEADER********************************************************************/

#include "Poisson.h"




CNPoisson::CNPoisson(double mean, CNRNG *gen)
    : CNRandom(gen)
{
    pMean = mean;
}

CNPoisson::CNPoisson()
	 : CNRandom((CNRNG*)NIL)
{
    pMean = 0;
}

CNPoisson::CNPoisson(CNParam *)
	 : CNRandom((CNRNG*)NIL)
{
    pMean = 0;
}

double CNPoisson::mean() { return pMean; }
double CNPoisson::mean(double x) {
    double t = pMean;
    pMean = x;
    return t;
}



double CNPoisson::operator()()
{
    double bound = exp(-1.0 * pMean);
    int count = 0;

    for (double product = 1.0;
     product >= bound;
     product *= pGenerator -> as_double()) {
         count++;
     }
    return(count - 1);
}



/***** Default I/O member function for CNCL classes **************************/

// CNNormal output
void CNPoisson::print(ostream &strm) const
{
    strm << "mean = " << pMean << endl;
}

// Debug output
void CNPoisson::dump(ostream &strm) const
{
    strm << "CNPoisson { "
	 << "mean = " << pMean
	 << " }" << endl;
}



/***** CNCL stuff for type information and exemplar objects ******************/

// Describing object for class CNPoisson
static CNClass CNPoisson_desc("CNPoisson", "$Revision: 0.29 $", CNPoisson::new_object);

// "Type" for type checking functions
CNClassDesc CN_POISSON = &CNPoisson_desc;






