//*BHEADER* :ts=8  -*- C++ -*-
/*****************************************************************************
 *
 *   |_|_|_  |_|_    |_    |_|_|_  |_		     C O M M U N I C A T I O N
 * |_        |_  |_  |_  |_        |_		               N E T W O R K S
 * |_        |_  |_  |_  |_        |_		                     C L A S S
 *   |_|_|_  |_    |_|_    |_|_|_  |_|_|_|_	                 L I B R A R Y
 *
 * $Id: RndInt.c,v 0.29 1995/07/05 12:47:34 cncl-adm Exp $
 *
 * CNClass: CNRndInt --- CNRandom Integers
 *
 *****************************************************************************
 * Copyright (C) 1992-1995   Communication Networks
 *                           Aachen University of Technology
 *                           D-52056 Aachen
 *                           Germany
 *                           Email: cncl-adm@dfv.rwth-aachen.de
 *****************************************************************************
 * This file is part of the CN class library. All files marked with
 * this header are free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.  This library is
 * distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.  You should have received a copy of the GNU
 * Library General Public License along with this library; if not, write
 * to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *****************************************************************************
 * original Copyright:
 * -------------------
 * Copyright (C) 1988 Free Software Foundation
 *    written by Dirk Grunwald (grunwald@cs.uiuc.edu)
 * 
 * This file is part of the GNU C++ Library.  This library is free
 * software; you can redistribute it and/or modify it under the terms of
 * the GNU Library General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  This library is distributed in the hope
 * that it will be useful, but WITHOUT ANY WARRANTY; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU Library General Public License for more details.
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 **EHEADER********************************************************************/


#include "RndInt.h"



CNRndInt::CNRndInt(long low, long high, CNRNG *gen) 
    : pLow((low < high) ? low : high),
      pHigh((low < high) ? high : low),
      pGenerator(gen)
{}

CNRndInt::CNRndInt(long high, CNRNG *gen) 
    : pLow((0 < high) ? 0 : high),
      pHigh((0 < high) ? high : 0),
      pGenerator(gen)
{}
  

CNRndInt::CNRndInt(CNRNG *gen)
    : pLow(0),
      pHigh(1),
      pGenerator(gen)

{}

CNRndInt::CNRndInt()
    : pLow(0),
      pHigh(0),
      pGenerator(NIL)
{}

CNRndInt::CNRndInt(CNParam *)
    : pLow(0),
      pHigh(0),
      pGenerator(NIL)
{}

CNRNG* CNRndInt::generator() const { return pGenerator;}
long CNRndInt::low() const       { return pLow; }
long CNRndInt::high() const      { return pHigh; }

CNRNG* CNRndInt::generator(CNRNG *gen) 
{
    CNRNG *tmp = pGenerator; pGenerator = gen;  return tmp;
}

long CNRndInt::low(long x)  
{
    long tmp = pLow;  pLow = x;  return tmp;
}

long CNRndInt:: high(long x) 
{
  long tmp = pHigh; pHigh = x; return tmp;
}



/***** Default I/O member function for CNCL classes **************************/

// CNNormal output
void CNRndInt::print(ostream &strm) const
{
    strm << "low = " << pLow << " high = " << pHigh << endl;
}

// Debug output
void CNRndInt::dump(ostream &strm) const
{
    strm << "CNRndInt { "
	 << "low = " << pLow << " high = " << pHigh
	 << " }" << endl;
}



/***** CNCL stuff for type information and exemplar objects ******************/

// Describing object for class CNRndInt
static CNClass CNRndInt_desc("CNRndInt", "$Revision: 0.29 $", CNRndInt::new_object);

// "Type" for type checking functions
CNClassDesc CN_RNDINT = &CNRndInt_desc;






